package org.springblade.modules.api.service.impl;

import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.alipay.api.AlipayApiException;
import com.alipay.api.AlipayClient;
import com.alipay.api.request.DatadigitalFincloudGeneralsaasFaceCertifyInitializeRequest;
import com.alipay.api.request.DatadigitalFincloudGeneralsaasFaceCertifyQueryRequest;
import com.alipay.api.request.DatadigitalFincloudGeneralsaasFaceCertifyVerifyRequest;
import com.alipay.api.response.DatadigitalFincloudGeneralsaasFaceCertifyInitializeResponse;
import com.alipay.api.response.DatadigitalFincloudGeneralsaasFaceCertifyQueryResponse;
import com.alipay.api.response.DatadigitalFincloudGeneralsaasFaceCertifyVerifyResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springblade.core.redis.cache.BladeRedis;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.modules.api.service.AliFaceService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.net.URLEncoder;
import java.util.HashMap;
import java.util.Map;

@Service
public class AliFaceServiceImpl implements AliFaceService {
	private static final Logger log = LoggerFactory.getLogger(AliFaceService.class);
	@Autowired
	private AlipayClient alipayClient;

	@Autowired
	private BladeRedis bladeRedis;


	/**1.初始化身份认证
	 * 初始化身份认证
	 *
	 * @param realName 真实姓名
	 * @param idNum    身份证号码
	 * @return
	 */
	public String getCertifyId(String realName, String idNum) {
		DatadigitalFincloudGeneralsaasFaceCertifyInitializeRequest request = new DatadigitalFincloudGeneralsaasFaceCertifyInitializeRequest();

		//构造身份信息json对象  JSONObject 阿里的fastjson
		JSONObject identityObj = new JSONObject();
		//身份类型，必填
		identityObj.put("identity_type", "CERT_INFO");
		//证件类型，必填
		identityObj.put("cert_type", "IDENTITY_CARD");
		//真实姓名，必填
		identityObj.put("cert_name", realName);
		//证件号码，必填
		identityObj.put("cert_no", idNum);

		//构造商户配置json对象
		JSONObject merchantConfigObj = new JSONObject();
		try {
			//认证完成后手机支付宝的返回页面，我这里返回首页
			String returnUrl = URLEncoder.encode("https://m.alipay.com/Gk8NF23", "UTF-8");
			// 设置回调地址,必填. 如果需要直接在支付宝APP里面打开回调地址使用alipay协议，参考下面的案例：appId用固定值 20000067，url替换为urlEncode后的业务回跳地址
			// alipays://platformapi/startapp?appId=20000067&url=https%3A%2F%2Fapp.cqkqinfo.com%2Fcertify%2FzmxyBackNew.do
			merchantConfigObj.put("return_url", "alipays://platformapi/startapp?appId=20000067&url=" + returnUrl);

			//构造身份认证初始化服务业务参数数据
			JSONObject bizContentObj = new JSONObject();
			//商户请求的唯一标识，推荐为uuid，必填  IdUtil hutool工具
			bizContentObj.put("outer_order_no", IdUtil.simpleUUID());
			bizContentObj.put("biz_code", "FACE");
			bizContentObj.put("identity_param", identityObj);
			bizContentObj.put("merchant_config", merchantConfigObj);
			request.setBizContent(bizContentObj.toString());

			//发起请求
			DatadigitalFincloudGeneralsaasFaceCertifyInitializeResponse response = alipayClient.certificateExecute(request);
			if (response.isSuccess()) {
				log.info("[身份验证]初始化身份认证成功:{}", realName);
				//接口调用成功，从返回对象中获取certify_id
				String certifyId = response.getCertifyId();
				// 生成的certifyId 是查询认证结果的凭证，需要保存下来，我这是保存到数据库中，也可保存到redis
			//	userMapper.insertCertifyId(idNum, certifyId);
				return certifyId;
			} else {
				log.info("[身份验证]初始化身份认证失败:{}", realName);
				return null;
			}
		} catch (Exception e) {
			log.error("[身份验证]初始化身份认证失败:{}", realName, e);
			return null;
		}
	}


	/**2.生成认证url
	 * 开始认证服务调用
	 *
	 * @param realName 真实姓名
	 * @param idNum    身份证号码
	 * @return         url-支付宝认证链接
	 */
	public String startCertify(String realName, String idNum) {
		String certifyId = getCertifyId(realName, idNum);
		if (StrUtil.isBlank(certifyId)) {
			return null;
		}

		DatadigitalFincloudGeneralsaasFaceCertifyVerifyRequest request = new DatadigitalFincloudGeneralsaasFaceCertifyVerifyRequest();
		//设置certifyId
		JSONObject bizContentObj = new JSONObject();
		bizContentObj.put("certify_id", certifyId);
		HashMap<String, Object> map = new HashMap<>();
		map.put("certify_id", certifyId);
		map.put("realName", realName);
		map.put("idNum", idNum);
		bladeRedis.setEx("ali_faceInfo:"+ AuthUtil.getUser().getUserId(),map,5*60L);
		request.setBizContent(bizContentObj.toString());

		//生成请求链接，这里一定要使用GET模式
		try {
			DatadigitalFincloudGeneralsaasFaceCertifyVerifyResponse response = alipayClient.certificateExecute(request);
			if (response.isSuccess()) {
				log.info("[身份验证]开始认证服务调用成功:{}", realName);
				return response.getBody();
			} else {
				log.info("[身份验证]开始认证服务调用失败:{}", realName);
				return null;
			}
		} catch (AlipayApiException e) {
			log.error("[身份验证]开始认证服务调用失败:{}", realName, e);
			return null;
		}
	}


	/**3.查询认证结果
	 * 查询身份认证结果
	 *
	 * @param certifyId 需要你在你的业务中获取之前保存下来的 certify_id
	 * @return
	 */
	public boolean queryCertifyResult() {
		String certifyId="";
		//获取当前登录用户的certify_id
		Object o = bladeRedis.get("ali_faceInfo:"+ AuthUtil.getUser().getUserId());
		if (ObjectUtil.isEmpty(o)){
			log.info("[身份验证]认证失败:{}", "当前登录用户certifyId为空");
			return false;
		}
		Map<String,Object> map = (Map) o;
		certifyId=map.get("certify_id").toString();
		DatadigitalFincloudGeneralsaasFaceCertifyQueryRequest request = new DatadigitalFincloudGeneralsaasFaceCertifyQueryRequest ();
		JSONObject bizContentObj = new JSONObject();
		bizContentObj.put("certify_id", certifyId);
		request.setBizContent(bizContentObj.toString());
		try {
			DatadigitalFincloudGeneralsaasFaceCertifyQueryResponse response = alipayClient.certificateExecute(request);
			if (response.isSuccess()&&"T".equalsIgnoreCase(response.getPassed())) {
				log.info("[身份验证]认证查询服务调用成功:{}", certifyId);
				return true;
			} else {
				log.info("[身份验证]认证查询服务调用失败:{}", certifyId);
			}
		} catch (AlipayApiException e) {
			log.error("[身份验证]开始认证服务调用失败:{}", certifyId, e.getMessage());
		}
		return false;
	}

}
