package org.springblade.modules.api.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.alipay.api.AlipayApiException;
import com.alipay.api.AlipayClient;
import com.alipay.api.request.AlipaySystemOauthTokenRequest;
import com.alipay.api.request.AlipayUserCertdocCertverifyConsultRequest;
import com.alipay.api.request.AlipayUserCertdocCertverifyPreconsultRequest;
import com.alipay.api.response.AlipaySystemOauthTokenResponse;
import com.alipay.api.response.AlipayUserCertdocCertverifyConsultResponse;
import com.alipay.api.response.AlipayUserCertdocCertverifyPreconsultResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springblade.common.utils.StringUtils;
import org.springblade.core.tool.api.R;
import org.springblade.modules.api.service.IAliService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class IAliServiceImpl implements IAliService {

	private static final Logger log = LoggerFactory.getLogger(IAliService.class);
	@Autowired
	private AlipayClient alipayClient;

	@Override
	public R getVerifyid(String realName, String idNum, String mobile) throws AlipayApiException {
		AlipayUserCertdocCertverifyPreconsultRequest request = new AlipayUserCertdocCertverifyPreconsultRequest();
		//构造身份信息json对象  JSONObject 阿里的fastjson
		JSONObject identityObj = new JSONObject();
		//真实姓名，必填
		identityObj.put("user_name", realName);
		//证件类型，必填
		identityObj.put("cert_type", "IDENTITY_CARD");
		//证件号码,必填
		identityObj.put("cert_no", idNum);
		//手机号
		identityObj.put("mobile", mobile);
		request.setBizContent(identityObj.toString());
		AlipayUserCertdocCertverifyPreconsultResponse response = alipayClient.certificateExecute(request);
		if (response.isSuccess()) {
			return R.data(response.getVerifyId());
		} else {
			return R.fail(response.getSubMsg());
		}
	}

	@Override
	public R realNameCheck(String verifyid, String authCode) throws AlipayApiException {
		String accessToken = this.getAccessTokenByAuthCode(authCode);
		if (StringUtils.isBlank(accessToken)) {
			return R.fail("获取令牌失败");
		}
		AlipayUserCertdocCertverifyConsultRequest request = new AlipayUserCertdocCertverifyConsultRequest();
		JSONObject jsonObject = new JSONObject();
		jsonObject.put("verify_id",verifyid);
		request.setBizContent(jsonObject.toString());
		AlipayUserCertdocCertverifyConsultResponse response = alipayClient.certificateExecute(request,accessToken);
		if(response.isSuccess()){
			String passed = response.getPassed();
			if ("T".equalsIgnoreCase(passed)){
				return R.success("验证成功");
			}else {
				log.info("支付宝真实信息验证失败参数:{}",response.getFailParams());
				log.info("支付宝真实信息验证失败原因:{}",response.getFailReason());
				return R.fail(response.getFailReason());
			}
		} else {
			return R.fail("支付宝真实信息验证失败");
		}
	}

	//根据authCode授权码获取accessToken令牌
	public String getAccessTokenByAuthCode(String authCode) {
		AlipaySystemOauthTokenRequest request = new AlipaySystemOauthTokenRequest();
		request.setCode(authCode);
		request.setGrantType("authorization_code");
		try {
			AlipaySystemOauthTokenResponse response = alipayClient.execute(request);
			if (response.isSuccess()) {
				String accessToken = response.getAccessToken();
				return accessToken;
			} else {
				throw new AlipayApiException(response.getSubMsg());
			}
		} catch (AlipayApiException e) {
			//处理异常
			e.printStackTrace();
			log.info("获取令牌失败:{}",e.getMessage());
		}
		return null;
	}
}
